#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "php.h"
#include "ext/standard/info.h"
#include "php_hasp.h"

#include <hasp.h>

static unsigned char hasp_scope[] = { // "<haspscope />"
   0xbc, 0xe8, 0xe1, 0xf3, 0xf0, 0xf3, 0xe3, 0xef, 0xf0, 0xe5, 0xa0, 0xaf,
   0xbe, 0x00
};
static unsigned char hasp_format[] = { // "<haspformat root=\"hasp\"><hasp><attribute name=\"id\" /></hasp></haspformat>"
  0xbc, 0xe8, 0xe1, 0xf3, 0xf0, 0xe6, 0xef, 0xf2, 0xed, 0xe1, 0xf4, 0xa0,
  0xf2, 0xef, 0xef, 0xf4, 0xbd, 0xa2, 0xf2, 0xe3, 0xf3, 0xa2, 0xbe, 0xbc,
  0xe8, 0xe1, 0xf3, 0xf0, 0xbe, 0xbc, 0xe1, 0xf4, 0xf4, 0xf2, 0xe9, 0xe2,
  0xf5, 0xf4, 0xe5, 0xa0, 0xee, 0xe1, 0xed, 0xe5, 0xbd, 0xa2, 0xe9, 0xe4,
  0xa2, 0xa0, 0xaf, 0xbe, 0xbc, 0xaf, 0xe8, 0xe1, 0xf3, 0xf0, 0xbe, 0xbc,
  0xaf, 0xe8, 0xe1, 0xf3, 0xf0, 0xe6, 0xef, 0xf2, 0xed, 0xe1, 0xf4, 0xbe,
  0x00
};
static unsigned char hasp_vendor[] = {
   0xb9, 0xd1, 0xf9, 0xaf, 0xe8, 0xf5, 0xb2, 0xb9, 0xcb, 0xf1, 0xe2, 0xe9, // "9Qy/hu29KqbiuC/M46LysyBHFJ0jckNEbMyeL66pfveITP1aGKzSjrHaMT0HVgVc/U4PCxwy1EIaJNJy"
   0xf5, 0xc3, 0xaf, 0xcd, 0xb4, 0xb6, 0xcc, 0xf9, 0xf3, 0xf9, 0xc2, 0xc8, // "YDBDSKo+8KZyM342qqusrut9zOwveblda44CsLnk5vOrYTxFpNBYMIQZrjkeA6Nclv//dCFVQsn/PUq3"
   0xc6, 0xca, 0xb0, 0xea, 0xe3, 0xeb, 0xce, 0xc5, 0xe2, 0xcd, 0xf9, 0xe5, // "06KRVnI8SKOwptsfFElbEb596KEKu/coF6gc7Fxj/8gbw/ulXCB70UNri73NzdfSt4j7vr7FT6p7d7RN"
   0xcc, 0xb6, 0xb6, 0xf0, 0xe6, 0xf6, 0xe5, 0xc9, 0xd4, 0xd0, 0xb1, 0xe1, // "xHIOfqEGoiXVc164TRzbHyw/bPgQtuoPtc+lIXyQtc6JHrjN1ujMXAPj6IyzABZiFFfgrp0BLwFf5cM3"
   0xc7, 0xcb, 0xfa, 0xd3, 0xea, 0xf2, 0xc8, 0xe1, 0xcd, 0xd4, 0xb0, 0xc8, // "k7UhyMZwiMpRvrT91UgnSqnpIpqRz/ViHu+0diBme7bTBRq79EHRl0Joxo6SzFDEzhTLFXKdnYX4Lws0"
   0xd6, 0xe7, 0xd6, 0xe3, 0xaf, 0xd5, 0xb4, 0xd0, 0xc3, 0xf8, 0xf7, 0xf9, // "KjxKtVr1aAygyTGc3QjeJYTlGRQSL85rV2bAZwvbTuRe31AwC1tKD3EvTdA+CyZ6gUFIsJXPT54nqpu0"
   0xb1, 0xc5, 0xc9, 0xe1, 0xca, 0xce, 0xca, 0xf9, 0xd9, 0xc4, 0xc2, 0xc4, // "nciGZqgAXplNn8Y71+ZWKqDli/yaqHWlnXYFQgOFiVrzvqpW48q8HxEVgHz3ran6CMadpFSFhFyNcslr"
   0xd3, 0xcb, 0xef, 0xab, 0xb8, 0xcb, 0xda, 0xf9, 0xcd, 0xb3, 0xb4, 0xb2, // "FWUdhMZdkxcpRFSpXZktpOXJAvLhmGa+i/wR8VM59AWBM73llg7B7QVoRm+gdcqzECpkVdmAqbXE+xQL"
   0xf1, 0xf1, 0xf5, 0xf3, 0xf2, 0xf5, 0xf4, 0xb9, 0xfa, 0xcf, 0xf7, 0xf6, // "kfsbVbkvgmS9G8O/3qU35iMKH2RcWUbZkB4ZZ9YX95h0xul1T4e5XT/RCY78MRh3qR8+9EiMJSu5ot20"
   0xe5, 0xe2, 0xec, 0xe4, 0xe1, 0xb4, 0xb4, 0xc3, 0xf3, 0xcc, 0xee, 0xeb, // "u5SEYWLApBWsuZEFlsx85GERFVV5c/7HCeB9LJvoy4h31PmjZHZEB4/r93tuk23VWWxSeMHS3/IUp4Y3"
   0xb5, 0xf6, 0xcf, 0xf2, 0xd9, 0xd4, 0xf8, 0xc6, 0xf0, 0xce, 0xc2, 0xd9, // "nc9gstrgsNjtlts9GJSzLT/xIDqXICKmUgCjd4h+hDB/dwC3wSknPuH+erO6+g/vd2VPG6uEuj3aQSP9"
   0xcd, 0xc9, 0xd1, 0xda, 0xf2, 0xea, 0xeb, 0xe5, 0xc1, 0xb6, 0xce, 0xe3, // "mGkMCMwOz3BUNHcQL9GOjK0X4PjE2IyE7P923O7cSjFm0nqaE8NDkzu4/Jg=";
   0xec, 0xf6, 0xaf, 0xaf, 0xe4, 0xc3, 0xc6, 0xd6, 0xd1, 0xf3, 0xee, 0xaf,
   0xd0, 0xd5, 0xf1, 0xb3, 0xb0, 0xb6, 0xcb, 0xd2, 0xd6, 0xee, 0xc9, 0xb8,
   0xd3, 0xcb, 0xcf, 0xf7, 0xf0, 0xf4, 0xf3, 0xe6, 0xc6, 0xc5, 0xec, 0xe2,
   0xc5, 0xe2, 0xb5, 0xb9, 0xb6, 0xcb, 0xc5, 0xcb, 0xf5, 0xaf, 0xe3, 0xef,
   0xc6, 0xb6, 0xe7, 0xe3, 0xb7, 0xc6, 0xf8, 0xea, 0xaf, 0xb8, 0xe7, 0xe2,
   0xf7, 0xaf, 0xf5, 0xec, 0xd8, 0xc3, 0xc2, 0xb7, 0xb0, 0xd5, 0xce, 0xf2,
   0xe9, 0xb7, 0xb3, 0xce, 0xfa, 0xe4, 0xe6, 0xd3, 0xf4, 0xb4, 0xea, 0xb7,
   0xf6, 0xf2, 0xb7, 0xc6, 0xd4, 0xb6, 0xf0, 0xb7, 0xe4, 0xb7, 0xd2, 0xce,
   0xf8, 0xc8, 0xc9, 0xcf, 0xe6, 0xf1, 0xc5, 0xc7, 0xef, 0xe9, 0xd8, 0xd6,
   0xe3, 0xb1, 0xb6, 0xb4, 0xd4, 0xd2, 0xfa, 0xe2, 0xc8, 0xf9, 0xf7, 0xaf,
   0xe2, 0xd0, 0xe7, 0xd1, 0xf4, 0xf5, 0xef, 0xd0, 0xf4, 0xe3, 0xab, 0xec,
   0xc9, 0xd8, 0xf9, 0xd1, 0xf4, 0xe3, 0xb6, 0xca, 0xc8, 0xf2, 0xea, 0xce,
   0xb1, 0xf5, 0xea, 0xcd, 0xd8, 0xc1, 0xd0, 0xea, 0xb6, 0xc9, 0xf9, 0xfa,
   0xc1, 0xc2, 0xda, 0xe9, 0xc6, 0xc6, 0xe6, 0xe7, 0xf2, 0xf0, 0xb0, 0xc2,
   0xcc, 0xf7, 0xc6, 0xe6, 0xb5, 0xe3, 0xcd, 0xb3, 0xeb, 0xb7, 0xd5, 0xe8,
   0xf9, 0xcd, 0xda, 0xf7, 0xe9, 0xcd, 0xf0, 0xd2, 0xf6, 0xf2, 0xd4, 0xb9,
   0xb1, 0xd5, 0xe7, 0xee, 0xd3, 0xf1, 0xee, 0xf0, 0xc9, 0xf0, 0xf1, 0xd2,
   0xfa, 0xaf, 0xd6, 0xe9, 0xc8, 0xf5, 0xab, 0xb0, 0xe4, 0xe9, 0xc2, 0xed,
   0xe5, 0xb7, 0xe2, 0xd4, 0xc2, 0xd2, 0xf1, 0xb7, 0xb9, 0xc5, 0xc8, 0xd2,
   0xec, 0xb0, 0xca, 0xef, 0xf8, 0xef, 0xb6, 0xd3, 0xfa, 0xc6, 0xc4, 0xc5,
   0xfa, 0xe8, 0xd4, 0xcc, 0xc6, 0xd8, 0xcb, 0xe4, 0xee, 0xd9, 0xd8, 0xb4,
   0xcc, 0xf7, 0xf3, 0xb0, 0xcb, 0xea, 0xf8, 0xcb, 0xf4, 0xd6, 0xf2, 0xb1,
   0xe1, 0xc1, 0xf9, 0xe7, 0xf9, 0xd4, 0xc7, 0xe3, 0xb3, 0xd1, 0xea, 0xe5,
   0xca, 0xd9, 0xd4, 0xec, 0xc7, 0xd2, 0xd1, 0xd3, 0xcc, 0xb8, 0xb5, 0xf2,
   0xd6, 0xb2, 0xe2, 0xc1, 0xda, 0xf7, 0xf6, 0xe2, 0xd4, 0xf5, 0xd2, 0xe5,
   0xb3, 0xb1, 0xc1, 0xf7, 0xc3, 0xb1, 0xf4, 0xcb, 0xc4, 0xb3, 0xc5, 0xf6,
   0xd4, 0xe4, 0xc1, 0xab, 0xc3, 0xf9, 0xda, 0xb6, 0xe7, 0xd5, 0xc6, 0xc9,
   0xf3, 0xca, 0xd8, 0xd0, 0xd4, 0xb5, 0xb4, 0xee, 0xf1, 0xf0, 0xf5, 0xb0,
   0xee, 0xe3, 0xe9, 0xc7, 0xda, 0xf1, 0xe7, 0xc1, 0xd8, 0xf0, 0xec, 0xce,
   0xee, 0xb8, 0xd9, 0xb7, 0xb1, 0xab, 0xda, 0xd7, 0xcb, 0xf1, 0xc4, 0xec,
   0xe9, 0xaf, 0xf9, 0xe1, 0xf1, 0xc8, 0xd7, 0xec, 0xee, 0xd8, 0xd9, 0xc6,
   0xd1, 0xe7, 0xcf, 0xc6, 0xe9, 0xd6, 0xf2, 0xfa, 0xf6, 0xf1, 0xf0, 0xd7,
   0xb4, 0xb8, 0xf1, 0xb8, 0xc8, 0xf8, 0xc5, 0xd6, 0xe7, 0xc8, 0xfa, 0xb3,
   0xf2, 0xe1, 0xee, 0xb6, 0xc3, 0xcd, 0xe1, 0xe4, 0xf0, 0xc6, 0xd3, 0xc6,
   0xe8, 0xc6, 0xf9, 0xce, 0xe3, 0xf3, 0xec, 0xf2, 0xc6, 0xd7, 0xd5, 0xe4,
   0xe8, 0xcd, 0xda, 0xe4, 0xeb, 0xf8, 0xe3, 0xf0, 0xd2, 0xc6, 0xd3, 0xf0,
   0xd8, 0xda, 0xeb, 0xf4, 0xf0, 0xcf, 0xd8, 0xca, 0xc1, 0xf6, 0xcc, 0xe8,
   0xed, 0xc7, 0xe1, 0xab, 0xe9, 0xaf, 0xf7, 0xd2, 0xb8, 0xd6, 0xcd, 0xb5,
   0xb9, 0xc1, 0xd7, 0xc2, 0xcd, 0xb7, 0xb3, 0xec, 0xec, 0xe7, 0xb7, 0xc2,
   0xb7, 0xd1, 0xd6, 0xef, 0xd2, 0xed, 0xab, 0xe7, 0xe4, 0xe3, 0xf1, 0xfa,
   0xc5, 0xc3, 0xf0, 0xeb, 0xd6, 0xe4, 0xed, 0xc1, 0xf1, 0xe2, 0xd8, 0xc5,
   0xab, 0xf8, 0xd1, 0xcc, 0xeb, 0xe6, 0xf3, 0xe2, 0xd6, 0xe2, 0xeb, 0xf6,
   0xe7, 0xed, 0xd3, 0xb9, 0xc7, 0xb8, 0xcf, 0xaf, 0xb3, 0xf1, 0xd5, 0xb3,
   0xb5, 0xe9, 0xcd, 0xcb, 0xc8, 0xb2, 0xd2, 0xe3, 0xd7, 0xd5, 0xe2, 0xda,
   0xeb, 0xc2, 0xb4, 0xda, 0xda, 0xb9, 0xd9, 0xd8, 0xb9, 0xb5, 0xe8, 0xb0,
   0xf8, 0xf5, 0xec, 0xb1, 0xd4, 0xb4, 0xe5, 0xb5, 0xd8, 0xd4, 0xaf, 0xd2,
   0xc3, 0xd9, 0xb7, 0xb8, 0xcd, 0xd2, 0xe8, 0xb3, 0xf1, 0xd2, 0xb8, 0xab,
   0xb9, 0xc5, 0xe9, 0xcd, 0xca, 0xd3, 0xf5, 0xb5, 0xef, 0xf4, 0xb2, 0xb0,
   0xf5, 0xb5, 0xd3, 0xc5, 0xd9, 0xd7, 0xcc, 0xc1, 0xf0, 0xc2, 0xd7, 0xf3,
   0xf5, 0xda, 0xc5, 0xc6, 0xec, 0xf3, 0xf8, 0xb8, 0xb5, 0xc7, 0xc5, 0xd2,
   0xc6, 0xd6, 0xd6, 0xb5, 0xe3, 0xaf, 0xb7, 0xc8, 0xc3, 0xe5, 0xc2, 0xb9,
   0xcc, 0xca, 0xf6, 0xef, 0xf9, 0xb4, 0xe8, 0xb3, 0xb1, 0xd0, 0xed, 0xea,
   0xda, 0xc8, 0xda, 0xc5, 0xc2, 0xb4, 0xaf, 0xf2, 0xb9, 0xb3, 0xf4, 0xf5,
   0xeb, 0xb2, 0xb3, 0xd6, 0xd7, 0xd7, 0xf8, 0xd3, 0xe5, 0xcd, 0xc8, 0xd3,
   0xb3, 0xaf, 0xc9, 0xd5, 0xf0, 0xb4, 0xd9, 0xb3, 0xee, 0xe3, 0xb9, 0xe7,
   0xf3, 0xf4, 0xf2, 0xe7, 0xf3, 0xce, 0xea, 0xf4, 0xec, 0xf4, 0xf3, 0xb9,
   0xc7, 0xca, 0xd3, 0xfa, 0xcc, 0xd4, 0xaf, 0xf8, 0xc9, 0xc4, 0xf1, 0xd8,
   0xc9, 0xc3, 0xcb, 0xed, 0xd5, 0xe7, 0xc3, 0xea, 0xe4, 0xb4, 0xe8, 0xab,
   0xe8, 0xc4, 0xc2, 0xaf, 0xe4, 0xf7, 0xc3, 0xb3, 0xf7, 0xd3, 0xeb, 0xee,
   0xd0, 0xf5, 0xc8, 0xab, 0xe5, 0xf2, 0xcf, 0xb6, 0xab, 0xe7, 0xaf, 0xf6,
   0xe4, 0xb2, 0xd6, 0xd0, 0xc7, 0xb6, 0xf5, 0xc5, 0xf5, 0xea, 0xb3, 0xe1,
   0xd1, 0xd3, 0xd0, 0xb9, 0xed, 0xc7, 0xeb, 0xcd, 0xc3, 0xcd, 0xf7, 0xcf,
   0xfa, 0xb3, 0xc2, 0xd5, 0xce, 0xc8, 0xe3, 0xd1, 0xcc, 0xb9, 0xc7, 0xcf,
   0xea, 0xcb, 0xb0, 0xd8, 0xb4, 0xd0, 0xea, 0xc5, 0xb2, 0xc9, 0xf9, 0xc5,
   0xb7, 0xd0, 0xb9, 0xb2, 0xb3, 0xcf, 0xb7, 0xe3, 0xd3, 0xea, 0xc6, 0xed,
   0xb0, 0xee, 0xf1, 0xe1, 0xc5, 0xb8, 0xce, 0xc4, 0xeb, 0xfa, 0xf5, 0xb4,
   0xaf, 0xca, 0xe7, 0xbd, 0x00
};

zend_function_entry hasp_functions[] = {
   {NULL, NULL, NULL}
};

zend_module_entry hasp_module_entry = {
   STANDARD_MODULE_HEADER,
   "hasp",
   hasp_functions,
   PHP_MINIT(hasp),
   NULL,
   NULL,
   NULL,
   NULL,
   HASP_VERSION,
   STANDARD_MODULE_PROPERTIES
};

#ifdef COMPILE_DL_HASP
ZEND_GET_MODULE(hasp)
#endif

static void hasp_unscramble(unsigned char *string);

PHP_MINIT_FUNCTION(hasp)
{
   hasp_status_t status;
   char *info = NULL, *start = NULL, *end = NULL;
   int ret = -1;

   hasp_unscramble(hasp_scope);
   hasp_unscramble(hasp_format);
   hasp_unscramble(hasp_vendor);

   do {
      status = hasp_get_info(hasp_scope, hasp_format, hasp_vendor, &info);
      if(status != HASP_STATUS_OK) {
         php_sleep(5);
      }
   } while(status != HASP_STATUS_OK);

   if(info) hasp_free(info);

   return SUCCESS;
}

static void hasp_unscramble(unsigned char *string)
{
   while(*string) {
      (*string) -= 128;
      string++;
   }

   return;
}
