#define emalloc(x) malloc(x)
#define efree(x) free(x)

#define TMP_BUF_SIZE 5

static unsigned char cipher_algo[] = "aes-256-cbc";
static unsigned char cipher_key[] = {
   0x6c, 0x39, 0x36, 0x07, 0x61, 0x56, 0x18, 0x70, 0x38, 0x31, 0x7f, 0x57,
   0x69, 0x01, 0x2a, 0x58, 0x1a, 0x6a, 0x22, 0x18, 0x18, 0x08, 0x18, 0x7e,
   0x45, 0x17, 0x6a, 0x15, 0x31, 0x4c, 0x1c, 0x1e
};
static unsigned char cipher_iv[] = {
   0x37, 0x3a, 0x35, 0x7e, 0x29, 0x51, 0x62, 0x5b, 0x62, 0x3e, 0x11, 0x03,
   0x76, 0x0c, 0x12, 0x17
};


#include <string.h>
#include <openssl/bio.h>
#include <openssl/evp.h>
#include <openssl/sha.h>

static int rcs_readphpfile(char *filename, unsigned char **buf);
static int rcs_readrcsfile(char *filename, unsigned char **buf);
static int rcs_writercsfile(char *filename, unsigned char *buf, unsigned int len);
int main(int argc, char *argv[]);

static int rcs_readphpfile(char *filename, unsigned char **buf)
{
   BIO *bio_out = NULL, *bio_mem = NULL, *bio_in = NULL, *bio_file = NULL;
   unsigned char *tmp = NULL;
   int len, ret = -1;

   do {
      if(!(bio_file = BIO_new(BIO_s_file()))) break;
      bio_in = bio_file;
      if(!BIO_read_filename(bio_file, filename)) break;

      if(!(bio_mem = BIO_new(BIO_s_mem()))) break;
      bio_out = bio_mem;

      if(!(tmp = emalloc(TMP_BUF_SIZE))) break;

      while((len = BIO_read(bio_in, tmp, TMP_BUF_SIZE)) > 0) {
         if(BIO_write(bio_out, tmp, len) != len) {
            len = -1;
            break;
         }
      }

      efree(tmp);

      if(len == -1) break;

      if((len = BIO_get_mem_data(bio_out, &tmp)) < 0) break;

      if(!(*buf = emalloc(len + 1))) break;

      memcpy(*buf, tmp, len);
      (*buf)[len] = '\0';

      ret = len;
   } while(0);

   if(bio_in) BIO_free_all(bio_in);

   if(bio_out) BIO_free_all(bio_out);

   return ret;
}

static int rcs_readrcsfile(char *filename, unsigned char **buf)
{
   BIO *bio_out = NULL, *bio_mem = NULL, *bio_in = NULL, *bio_file = NULL, *bio_base64 = NULL, *bio_cipher = NULL;
   unsigned char *tmp = NULL, hash[SHA_DIGEST_LENGTH];
   int len, ret = -1;

   OpenSSL_add_all_ciphers();

   do {
      if(!(bio_file = BIO_new(BIO_s_file()))) break;
      bio_in = bio_file;
      if(!BIO_read_filename(bio_file, filename)) break;

      if(!(bio_base64 = BIO_new(BIO_f_base64()))) break;
      bio_in = BIO_push(bio_base64, bio_in);

      if(!(bio_cipher = BIO_new(BIO_f_cipher()))) break;
      bio_in = BIO_push(bio_cipher, bio_in);
      BIO_set_cipher(bio_cipher, EVP_get_cipherbyname((char *)cipher_algo), cipher_key, cipher_iv, 0);

      if(!(bio_mem = BIO_new(BIO_s_mem()))) break;
      bio_out = bio_mem;

      if(!(tmp = emalloc(TMP_BUF_SIZE))) break;

      while((len = BIO_read(bio_in, tmp, TMP_BUF_SIZE)) > 0) {
         if(BIO_write(bio_out, tmp, len) != len) {
            len = -1;
            break;
         }
      }

      efree(tmp);

      if(len == -1) break;

      if((len = BIO_get_mem_data(bio_out, &tmp)) < SHA_DIGEST_LENGTH) break;

      len -= SHA_DIGEST_LENGTH;

      SHA1(tmp, len, hash);
      if(memcmp(&tmp[len], hash, SHA_DIGEST_LENGTH)) break;

      if(!(*buf = emalloc(len + 1))) break;

      memcpy(*buf, tmp, len);
      (*buf)[len] = '\0';

      ret = len;
   } while(0);

   if(bio_in) BIO_free_all(bio_in);

   if(bio_out) BIO_free_all(bio_out);

   EVP_cleanup();

   return ret;
}

static int rcs_writercsfile(char *filename, unsigned char *buf, unsigned int len)
{
   BIO *bio_out = NULL, *bio_file = NULL, *bio_base64 = NULL, *bio_cipher = NULL;
   unsigned char hash[SHA_DIGEST_LENGTH];
   int ret = -1;

   OpenSSL_add_all_ciphers();

   do {
      if(!(bio_file = BIO_new(BIO_s_file()))) break;
      bio_out = bio_file;
      if(!BIO_write_filename(bio_file, filename)) break;

      if(!(bio_base64 = BIO_new(BIO_f_base64()))) break;
      bio_out = BIO_push(bio_base64, bio_out);

      if(!(bio_cipher = BIO_new(BIO_f_cipher()))) break;
      bio_out = BIO_push(bio_cipher, bio_out);
      BIO_set_cipher(bio_cipher, EVP_get_cipherbyname((char *)cipher_algo), cipher_key, cipher_iv, 1);

      SHA1(buf, len, hash);

      if(BIO_write(bio_out, buf, len) != len) break;
      if(BIO_write(bio_out, hash, SHA_DIGEST_LENGTH) != SHA_DIGEST_LENGTH) break;

      if(BIO_flush(bio_out) != 1) break;

      ret = 0;
   } while(0);

   if(bio_out) BIO_free_all(bio_out);

   EVP_cleanup();

   return ret;
}

int main(int argc, char *argv[])
{
   unsigned char *buf = NULL, *start = NULL, *end = NULL;
   int len;

   if(argc < 3) {
      printf("usage: %s <phpfile> <rcsfile>\n", argv[0]);
      return -1;
   }

   len = rcs_readphpfile(argv[1], &buf);

   if((start = (unsigned char *)strstr((char *)buf, "<?php")) == NULL) {
      return -1;
   }

   start += strlen("<?php");

   if(((end = (unsigned char *)strrchr((char *)start, '?')) == NULL) || (end[1] != '>')) {
      return -1;
   }

   end[0] = '\0';

   rcs_writercsfile(argv[2], start, strlen((char *)start));

   return 0;
}
